%% CREATE_WSJ - script to generate noised-up version of WSJ
%

%% Introduction
% |renoiser| is a tool to analyze the channel of clean speech 
% in a filtered, noisy channel, and optionally to recombine the 
% same or different speech, at any SNR, using the extracted channel 
% filter, with the residual noise.  This script shows how to
% generate a filtered- and noised-up version of an existing speech 
% corpus.  Specifically, we use the RATS rebroadcast example
% signals (LDC2011E20) to estimate some noise/filter
% characteristics, then apply them to one directory from the WSJ 
% speech corpus.
%
% First, we analyze each of the channel charactersitcs:

droot = ['../../data/LDC2011E20/data/default/' ...
         '20110316_145021_recvrcali_default_'];
tst = 8.6;
tend = 16.0;
list = '4bj1-2.txt';
mixoutbase = 'WSJ/ch';
% There's really no energy above 4 kHz
TARGETSR = 8000;

% all 8 channels
for chan = 'ABCDEFGH'

%% Analysis
%

% Create a per-channel output directory
dirname = [mixoutbase,chan];
mkdir(dirname);
noisename = fullfile(dirname,'noise.wav');
filtername = fullfile(dirname,'filter.wav');

% Analyze the channel, remember the reported SNR
[d,sr,SNR,fshift] =  renoiser('-clean', [droot,'REF.flac'], ...
                       '-mix', [droot,chan, '.flac'], ...
                       '-disp', 1, '-targetsr', TARGETSR, ...
                       '-noisefloor', '-30', ...
                       '-checkfshift', '1', ...
                       '-start', tst, '-end', tend, ...
                       '-targetout', fullfile(dirname,'target.wav'), ...
                       '-noiseout', noisename, ...
                       '-filterout', filtername);

  
%% Synthesis
%

  % smooth background noise over a 1s window to remove
  % noise-correlated features
  laundersecs = 1.0;

  % Process a directory full of WSJ files, use the SNR from analysis
  renoiser('-cleanlist', list, '-mixoutdir', dirname, ...
           '-noise', noisename, '-filter', filtername, ...
           '-laundernoise', laundersecs, '-SNR', SNR, '-fshift', fshift);

  % also, extract the original for reference
  [d,sr] = audioread([droot,chan,'.flac'],TARGETSR);
  wavwrite(d(round(tst*sr)+[1:round((tend-tst)*sr)]),sr,...
           fullfile(dirname,'orig.wav'));

end

%% Results
%
% You can listen to the results in the following table:
%
% <html>
% <table border=1>
% <tr><td>&nbsp;</td><td>Original</td><td>Target</td>
% <td>Noise</td><td>Renoised</td></tr>
% <tr><td>Chan A</td>
% <td><a href="WSJ/chA/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chA/target.wav">target.wav</a></td>
% <td><a href="WSJ/chA/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chA/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan B</td>
% <td><a href="WSJ/chB/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chB/target.wav">target.wav</a></td>
% <td><a href="WSJ/chB/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chB/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan C</td>
% <td><a href="WSJ/chC/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chC/target.wav">target.wav</a></td>
% <td><a href="WSJ/chC/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chC/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan D</td>
% <td><a href="WSJ/chD/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chD/target.wav">target.wav</a></td>
% <td><a href="WSJ/chD/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chD/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan E</td>
% <td><a href="WSJ/chE/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chE/target.wav">target.wav</a></td>
% <td><a href="WSJ/chE/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chE/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan F</td>
% <td><a href="WSJ/chF/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chF/target.wav">target.wav</a></td>
% <td><a href="WSJ/chF/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chF/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan G</td>
% <td><a href="WSJ/chG/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chG/target.wav">target.wav</a></td>
% <td><a href="WSJ/chG/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chG/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% <tr><td>Chan H</td>
% <td><a href="WSJ/chH/orig.wav">orig.wav</a></td>
% <td><a href="WSJ/chH/target.wav">target.wav</a></td>
% <td><a href="WSJ/chH/noise.wav">noise.wav</a></td>
% <td><a href="WSJ/chH/4bja0101.wav">4bja0101.wav</a></td>
% </tr>
% </table>
% </html>